@@ -70,16 +70,10 @@ from griffe.expressions import Expression, Name
 from griffe.logger import LogLevel
 
 if TYPE_CHECKING:
-    import sys
-    from typing import Any, Pattern
+    from typing import Any, Literal, Pattern
 
     from griffe.dataclasses import Docstring
 
-    if sys.version_info < (3, 8):
-        from typing_extensions import Literal
-    else:
-        from typing import Literal
-
 
 _warn = warning(__name__)
 
@@ -105,7 +99,13 @@ def _is_dash_line(line: str) -> bool:
     return not _is_empty_line(line) and _is_empty_line(line.replace("-", ""))
 
 
-def _read_block_items(docstring: Docstring, *, offset: int) -> tuple[list[list[str]], int]:
+def _read_block_items(
+    docstring: Docstring,
+    *,
+    offset: int,
+    allow_section_blank_line: bool,
+    **options: Any,  # noqa: ARG001
+) -> tuple[list[list[str]], int]:
     lines = docstring.lines
     if offset >= len(lines):
         return [], offset
@@ -129,7 +129,7 @@ def _read_block_items(docstring: Docstring, *, offset: int) -> tuple[list[list[s
 
         if line.startswith(4 * " "):
             # continuation line
-            current_item.append(line.lstrip())
+            current_item.append(line[4:])
             previous_was_empty = False
 
         elif line.startswith(" "):
@@ -145,14 +145,24 @@ def _read_block_items(docstring: Docstring, *, offset: int) -> tuple[list[list[s
             previous_was_empty = False
 
         elif _is_empty_line(line):
+            # two line breaks indicate the start of a new section
+            if previous_was_empty:
+                break
+
             # empty line: preserve it in the current item
             current_item.append("")
             previous_was_empty = True
 
         else:
-            if previous_was_empty:
+            # preserve original behavior, that a single line break between block
+            # items triggers a new section
+            if not allow_section_blank_line and previous_was_empty:
                 break
-            # new item
+
+            # detect the start of a new section
+            if new_offset + 1 < len(lines) and lines[new_offset + 1].startswith("---"):
+                break
+
             items.append(current_item)
             current_item = [line]
             previous_was_empty = False
@@ -201,7 +211,9 @@ _RE_RETURNS: Pattern = re.compile(
         |  # or
         (?P<name>{_RE_NAME})\s*:\s*  # just name
         |  # or
-        (?P<type>{_RE_TYPE})\s*  # just type
+        \s*:\s*$  # no name, no type
+        |  # or
+        (?::\s*)?(?P<type>{_RE_TYPE})\s*  # just type
     )
     """,
     re.IGNORECASE | re.VERBOSE,
@@ -230,11 +242,12 @@ def _read_parameters(
     *,
     offset: int,
     warn_unknown_params: bool = True,
+    **options: Any,
 ) -> tuple[list[DocstringParameter], int]:
     parameters = []
     annotation: str | Name | Expression | None
 
-    items, new_offset = _read_block_items(docstring, offset=offset)
+    items, new_offset = _read_block_items(docstring, offset=offset, **options)
 
     for item in items:
         match = _RE_PARAMETER.match(item[0])
@@ -267,7 +280,7 @@ def _read_parameters(
             else:
                 _warn(docstring, new_offset, f"No types or annotations for parameters {names}")
         else:
-            annotation = parse_annotation(annotation, docstring, log_level=LogLevel.debug)  # type: ignore[arg-type]
+            annotation = parse_annotation(annotation, docstring, log_level=LogLevel.debug)
 
         if default is None:
             for name in names:
@@ -297,9 +310,9 @@ def _read_parameters_section(
     docstring: Docstring,
     *,
     offset: int,
-    **options: Any,  # noqa: ARG001
+    **options: Any,
 ) -> tuple[DocstringSectionParameters | None, int]:
-    parameters, new_offset = _read_parameters(docstring, offset=offset)
+    parameters, new_offset = _read_parameters(docstring, offset=offset, **options)
 
     if parameters:
         return DocstringSectionParameters(parameters), new_offset
@@ -312,9 +325,9 @@ def _read_other_parameters_section(
     docstring: Docstring,
     *,
     offset: int,
-    **options: Any,  # noqa: ARG001
+    **options: Any,
 ) -> tuple[DocstringSectionOtherParameters | None, int]:
-    parameters, new_offset = _read_parameters(docstring, offset=offset, warn_unknown_params=False)
+    parameters, new_offset = _read_parameters(docstring, offset=offset, warn_unknown_params=False, **options)
 
     if parameters:
         return DocstringSectionOtherParameters(parameters), new_offset
@@ -327,12 +340,12 @@ def _read_deprecated_section(
     docstring: Docstring,
     *,
     offset: int,
-    **options: Any,  # noqa: ARG001
+    **options: Any,
 ) -> tuple[DocstringSectionDeprecated | None, int]:
     # deprecated
     # SINCE_VERSION
     #     TEXT?
-    items, new_offset = _read_block_items(docstring, offset=offset)
+    items, new_offset = _read_block_items(docstring, offset=offset, **options)
 
     if not items:
         _warn(docstring, new_offset, f"Empty deprecated section at line {offset}")
@@ -351,11 +364,11 @@ def _read_returns_section(
     docstring: Docstring,
     *,
     offset: int,
-    **options: Any,  # noqa: ARG001
+    **options: Any,
 ) -> tuple[DocstringSectionReturns | None, int]:
     # (NAME : )?TYPE
     #     TEXT?
-    items, new_offset = _read_block_items(docstring, offset=offset)
+    items, new_offset = _read_block_items(docstring, offset=offset, **options)
 
     if not items:
         _warn(docstring, new_offset, f"Empty returns section at line {offset}")
@@ -398,7 +411,7 @@ def _read_returns_section(
                         else:
                             annotation = return_item
         else:
-            annotation = parse_annotation(annotation, docstring, log_level=LogLevel.debug)  # type: ignore[arg-type]
+            annotation = parse_annotation(annotation, docstring, log_level=LogLevel.debug)
         returns.append(DocstringReturn(name=name or "", annotation=annotation, description=text))
     return DocstringSectionReturns(returns), new_offset
 
@@ -407,12 +420,12 @@ def _read_yields_section(
     docstring: Docstring,
     *,
     offset: int,
-    **options: Any,  # noqa: ARG001
+    **options: Any,
 ) -> tuple[DocstringSectionYields | None, int]:
     # yields
     # (NAME : )?TYPE
     #     TEXT?
-    items, new_offset = _read_block_items(docstring, offset=offset)
+    items, new_offset = _read_block_items(docstring, offset=offset, **options)
 
     if not items:
         _warn(docstring, new_offset, f"Empty yields section at line {offset}")
@@ -446,7 +459,7 @@ def _read_yields_section(
                 else:
                     annotation = yield_item
         else:
-            annotation = parse_annotation(annotation, docstring, log_level=LogLevel.debug)  # type: ignore[arg-type]
+            annotation = parse_annotation(annotation, docstring, log_level=LogLevel.debug)
         yields.append(DocstringYield(name=name or "", annotation=annotation, description=text))
     return DocstringSectionYields(yields), new_offset
 
@@ -455,12 +468,12 @@ def _read_receives_section(
     docstring: Docstring,
     *,
     offset: int,
-    **options: Any,  # noqa: ARG001
+    **options: Any,
 ) -> tuple[DocstringSectionReceives | None, int]:
     # receives
     # (NAME : )?TYPE
     #     TEXT?
-    items, new_offset = _read_block_items(docstring, offset=offset)
+    items, new_offset = _read_block_items(docstring, offset=offset, **options)
 
     if not items:
         _warn(docstring, new_offset, f"Empty receives section at line {offset}")
@@ -490,7 +503,7 @@ def _read_receives_section(
                     else:
                         annotation = receives_item
         else:
-            annotation = parse_annotation(annotation, docstring, log_level=LogLevel.debug)  # type: ignore[arg-type]
+            annotation = parse_annotation(annotation, docstring, log_level=LogLevel.debug)
         receives.append(DocstringReceive(name=name or "", annotation=annotation, description=text))
     return DocstringSectionReceives(receives), new_offset
 
@@ -499,12 +512,12 @@ def _read_raises_section(
     docstring: Docstring,
     *,
     offset: int,
-    **options: Any,  # noqa: ARG001
+    **options: Any,
 ) -> tuple[DocstringSectionRaises | None, int]:
     # raises
     # EXCEPTION
     #     TEXT?
-    items, new_offset = _read_block_items(docstring, offset=offset)
+    items, new_offset = _read_block_items(docstring, offset=offset, **options)
 
     if not items:
         _warn(docstring, new_offset, f"Empty raises section at line {offset}")
@@ -522,12 +535,12 @@ def _read_warns_section(
     docstring: Docstring,
     *,
     offset: int,
-    **options: Any,  # noqa: ARG001
+    **options: Any,
 ) -> tuple[DocstringSectionWarns | None, int]:
     # warns
     # WARNING
     #     TEXT?
-    items, new_offset = _read_block_items(docstring, offset=offset)
+    items, new_offset = _read_block_items(docstring, offset=offset, **options)
 
     if not items:
         _warn(docstring, new_offset, f"Empty warns section at line {offset}")
@@ -545,12 +558,12 @@ def _read_attributes_section(
     docstring: Docstring,
     *,
     offset: int,
-    **options: Any,  # noqa: ARG001
+    **options: Any,
 ) -> tuple[DocstringSectionAttributes | None, int]:
     # attributes (for classes)
     # NAME( : TYPE)?
     #    TEXT?
-    items, new_offset = _read_block_items(docstring, offset=offset)
+    items, new_offset = _read_block_items(docstring, offset=offset, **options)
 
     if not items:
         _warn(docstring, new_offset, f"Empty attributes section at line {offset}")
@@ -571,7 +584,7 @@ def _read_attributes_section(
             with suppress(AttributeError, KeyError):
                 annotation = docstring.parent.members[name].annotation  # type: ignore[union-attr]
         else:
-            annotation = parse_annotation(annotation, docstring, log_level=LogLevel.debug)  # type: ignore[arg-type]
+            annotation = parse_annotation(annotation, docstring, log_level=LogLevel.debug)
         text = dedent("\n".join(item[1:]))
         attributes.append(DocstringAttribute(name=name, annotation=annotation, description=text))
     return DocstringSectionAttributes(attributes), new_offset
@@ -659,6 +672,7 @@ def parse(
     *,
     ignore_init_summary: bool = False,
     trim_doctest_flags: bool = True,
+    allow_section_blank_line: bool = False,
     **options: Any,
 ) -> list[DocstringSection]:
     """Parse a docstring.
@@ -670,6 +684,9 @@ def parse(
         docstring: The docstring to parse.
         ignore_init_summary: Whether to ignore the summary in `__init__` methods' docstrings.
         trim_doctest_flags: Whether to remove doctest flags from Python example blocks.
+        allow_section_blank_line: Whether to continue a section if there's an empty line
+            between items in a formatted block, like Parameters or Returns.
+            If True, you can still create a new section using two empty lines.
         **options: Additional parsing options.
 
     Returns:
@@ -684,6 +701,7 @@ def parse(
     options = {
         "trim_doctest_flags": trim_doctest_flags,
         "ignore_init_summary": ignore_init_summary,
+        "allow_section_blank_line": allow_section_blank_line,
         **options,
     }
 
@@ -729,3 +747,6 @@ def parse(
         sections.append(DocstringSectionText("\n".join(current_section).rstrip("\n")))
 
     return sections
+
+
+__all__ = ["parse"]
