@@ -5,7 +5,7 @@ from __future__ import annotations
 import sys
 from contextlib import contextmanager
 from importlib import import_module
-from typing import TYPE_CHECKING, Any, Iterator
+from typing import TYPE_CHECKING, Any, Iterator, Sequence
 
 if TYPE_CHECKING:
     from pathlib import Path
@@ -33,7 +33,7 @@ def sys_path(*paths: str | Path) -> Iterator[None]:
         sys.path = old_path
 
 
-def dynamic_import(import_path: str, import_paths: list[Path] | None = None) -> Any:
+def dynamic_import(import_path: str, import_paths: Sequence[str | Path] | None = None) -> Any:
     """Dynamically import the specified object.
 
     It can be a module, class, method, function, attribute,
@@ -86,3 +86,6 @@ def dynamic_import(import_path: str, import_paths: list[Path] | None = None) ->
         except AttributeError as error:
             raise ImportError("\n".join(errors)) from error
     return value
+
+
+__all__ = ["dynamic_import", "sys_path"]
