@@ -2,8 +2,7 @@
 
 from __future__ import annotations
 
-import sys
-from typing import TYPE_CHECKING, Any, Iterator, List, Tuple, Union
+from typing import TYPE_CHECKING, Any, Iterator, List, Protocol, Tuple, Union
 
 from griffe.dataclasses import Attribute, Class, Docstring, Function, Module
 from griffe.docstrings.dataclasses import DocstringAttribute, DocstringElement, DocstringParameter, DocstringSection
@@ -12,11 +11,6 @@ if TYPE_CHECKING:
     from types import ModuleType
 
 
-if sys.version_info < (3, 8):
-    from typing_extensions import Protocol
-else:
-    from typing import Protocol
-
 ParentType = Union[Module, Class, Function, Attribute, None]
 ParseResultType = Tuple[List[DocstringSection], List[str]]
 
@@ -42,7 +36,7 @@ def parser(parser_module: ModuleType) -> Iterator[ParserType]:
     """
     original_warn = parser_module._warn
 
-    def parse(docstring: str, parent: ParentType = None, **parser_opts: Any) -> ParseResultType:
+    def parse(docstring: str, parent: ParentType | None = None, **parser_opts: Any) -> ParseResultType:
         """Parse a doctring.
 
         Parameters:
@@ -63,7 +57,7 @@ def parser(parser_module: ModuleType) -> Iterator[ParserType]:
         sections = parser_module.parse(docstring_object, **parser_opts)
         return sections, warnings
 
-    yield parse  # type: ignore[misc]
+    yield parse
 
     parser_module._warn = original_warn  # type: ignore[attr-defined]
 
@@ -98,5 +92,10 @@ def assert_element_equal(actual: DocstringElement, expected: DocstringElement) -
         actual: The actual element.
         expected: The expected element.
     """
-    assert actual.annotation == expected.annotation  # type: ignore[operator]
-    assert actual.description == expected.description
+    assert isinstance(actual, type(expected))
+
+    for k in actual.as_dict():
+        src = getattr(actual, k)
+        dst = getattr(expected, k)
+
+        assert src == dst, f"attribute {k!r}, {src!r} != {dst!r}"
