@@ -108,6 +108,24 @@ def test_empty_indented_lines_in_section_with_items(parse_numpy: ParserType) ->
     assert len(sections) == 2
     assert len(sections[0].value) == 1
 
+    # allow_section_blank_line requires at least 2 newlines to create a new section
+    sections2, _ = parse_numpy(docstring, allow_section_blank_line=True)
+    assert len(sections2) == 1
+    assert len(sections2[0].value) == 2
+
+
+def test_doubly_indented_lines_in_section_items(parse_numpy: ParserType) -> None:
+    """In sections with items, don't remove all spaces on the left of indented lines.
+
+    Parameters:
+        parse_numpy: Fixture parser.
+    """
+    docstring = "Returns\n-------\nonly_item : type\n    Description:\n\n    - List item.\n        - Sublist item."
+    sections, _ = parse_numpy(docstring)
+    assert len(sections) == 1
+    lines = sections[0].value[0].description.split("\n")
+    assert lines[-1].startswith(4 * " " + "- ")
+
 
 # =============================================================================================
 # Annotations
@@ -349,6 +367,12 @@ def test_returns_section(parse_numpy: ParserType) -> None:
         flag : bool
             Some kind
             of flag.
+        x :
+            Name only
+        :
+            No name or annotation
+        : int
+            Only annotation
     """
 
     sections, _ = parse_numpy(docstring)
@@ -359,7 +383,22 @@ def test_returns_section(parse_numpy: ParserType) -> None:
     )
     assert_element_equal(
         sections[0].value[1],
-        DocstringReturn(name="", annotation="bool", description="Some kind\nof flag."),
+        DocstringReturn(name="flag", annotation="bool", description="Some kind\nof flag."),
+    )
+
+    assert_element_equal(
+        sections[0].value[2],
+        DocstringReturn(name="x", annotation=None, description="Name only"),
+    )
+
+    assert_element_equal(
+        sections[0].value[3],
+        DocstringReturn(name="", annotation=None, description="No name or annotation"),
+    )
+
+    assert_element_equal(
+        sections[0].value[4],
+        DocstringReturn(name="", annotation="int", description="Only annotation"),
     )
 
 
@@ -387,7 +426,7 @@ def test_yields_section(parse_numpy: ParserType) -> None:
     )
     assert_element_equal(
         sections[0].value[1],
-        DocstringYield(name="", annotation="bool", description="Some kind\nof flag."),
+        DocstringYield(name="flag", annotation="bool", description="Some kind\nof flag."),
     )
 
 
@@ -415,7 +454,7 @@ def test_receives_section(parse_numpy: ParserType) -> None:
     )
     assert_element_equal(
         sections[0].value[1],
-        DocstringReceive(name="", annotation="bool", description="Some kind\nof flag."),
+        DocstringReceive(name="flag", annotation="bool", description="Some kind\nof flag."),
     )
 
 
